{-# LANGUAGE DataKinds #-}
{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE TypeOperators #-}
{-# LANGUAGE OverloadedStrings #-}

module HAL.Client where

import Data.Proxy
import GHC.Generics
import Servant.API
import Servant.Client hiding (Response)
import Data.Text
import Data.Map
import Data.Aeson

type HALAPI = Search

type Search = "search"
  -- fl determine which fields will be returned it can be a list of fields or *
  :> QueryParams "fl" Text
  -- TODO: type this monster
  -- fq is to filter request
  :> QueryParams "fq" Text
  -- pretty much clear, (Asc || Desc) + field you want to sort by
  :> QueryParam "sort" SortField
  -- permit to start a the x result
  :> QueryParam "start" Int
  -- use rows to make the request only return the x number of result
  :> QueryParam "rows" Int
  :> Get '[JSON] Response

-- Get's argument type
data SortField = Asc Text | Desc Text
  deriving (Show)
instance ToHttpApiData SortField where
  toUrlPiece (Asc t) = t <> " asc"
  toUrlPiece (Desc t) = t <> " desc"

asc :: Text -> Maybe SortField
asc = Just . Asc

desc :: Text -> Maybe SortField
desc = Just . Desc

-- Response type
data Response = Response
  {
    numFound :: Integer,
    start :: Int,
    docs :: [Doc]
  } deriving (Show, Generic)

instance FromJSON Response where
  parseJSON (Object o) = Response <$>
                         ((o .: "response") >>= (.: "numFound"))
                         <*> ((o .: "response") >>= (.: "start"))
                         <*> ((o .: "response") >>= (.: "docs"))

newtype Doc = Doc (Map Text Value)
  deriving (Show, Generic)
instance FromJSON Doc

halAPI :: Proxy HALAPI
halAPI = Proxy

search :: [Text]
  -> [Text]
  -> Maybe SortField
  -> Maybe Int
  -> Maybe Int
  -> ClientM Response
search = client halAPI
